/*****************************************************************************
 *  File: app_selector.h
 * 
 *  This file contains the constants, structure definitions, and funtion
 *  prototypes for the application selector.
 *
 ****************************************************************************/

#ifndef APP_SELECTOR_H_
#define APP_SELECTOR_H_
#include "alt_video_display.h"

#define ALT_VIDEO_DISPLAY_COLS 800
#define ALT_VIDEO_DISPLAY_ROWS 480
#define ALT_VIDEO_DISPLAY_COLOR_DEPTH 32
#define ALT_VIDEO_DISPLAY_SGDMA_NAME LCD_SGDMA_NAME

#define NUM_VIDEO_FRAMES 2

#define RECONFIG_ADDRESS (64*65536)
#define EPCS_BLOCK_SIZE (65536*2)//

#define COLOR_R 0xFF0000
#define COLOR_G 0x00FF00
#define COLOR_B 0x0000FF
#define COLOR_WHITE 0xFFFFFF
#define COLOR_BLACK 0x000000
#define COLOR_GRAY  0x7F7F7F

// Flash memory map
// =====================================================
// Offset                Contents
// =====================================================
// 0x000000 - 0x007FFF   Application Boot Code
// 0x008000 - 0x00FFFF   HW Image Catalog
// 0x010000 - 0x01FFFF   Unused
// 0x020000 - 0x0FFFFF   Selector HW Image
// 0x100000 - 0x23FFFF   Selector SW Image
// 0x240000 - 0x63FFFF   Application SW
// 0x640000 - 0xDBFFFF   Application HW Images (10)
// 0xDC0000 - 0xFFFFFF   Unused


#define AS_FLASH_SECTOR_SIZE            0x20000  //128KB
#define AS_SIZE_3C25_SOF                703693
#define AS_SECTORS_FOR_HW_IMAGE         ((AS_SIZE_3C25_SOF / AS_FLASH_SECTOR_SIZE) + 1)
#define AS_BYTES_FOR_HW_IMAGE           (AS_SECTORS_FOR_HW_IMAGE * AS_FLASH_SECTOR_SIZE)

#define AS_FLASH_IMAGE_CATALOG_OFFSET   0x8000
#define AS_FLASH_IMAGE_CATALOG_SIZE     0x8000
#define AS_HW_IMAGE_OFFSET_START        0x640000
#define AS_HW_IMAGE_OFFSET_END          0xDC0000
#define AS_MAX_FLASH_IMAGES             ((AS_HW_IMAGE_OFFSET_END - AS_HW_IMAGE_OFFSET_START) / AS_BYTES_FOR_HW_IMAGE)

#define AS_SW_IMAGE_OFFSET              0x240000

#define AS_SREC_BUFFER_SIZE             0x20000

// Error Codes
#define AS_SUCCESS                      0
#define AS_FAILURE                     -1
#define AS_ERROR_FAT_FAILED_TO_MOUNT   -2
#define AS_ERROR_FAT_COULD_NOT_FIND    -3
#define AS_ERROR_FAT_READ_ZERO_BYTES   -4
#define AS_ERROR_SREC_HAS_NO_TIMESTAMP -5
#define AS_ERROR_COULD_NOT_ALLOCATE    -6

#define AS_MAX_SDCARD_APPS              40

// Main menu actions
#define AS_ACTION_NONE                  0
#define AS_ACTION_LOAD_APP              1
#define AS_ACTION_DISPLAY_APP_INFO      2
#define AS_ACTION_APP_SELECTOR_EXIT     3

// Confirm Window actions
#define AS_ACTION_CANCEL                0
#define AS_ACTION_OK                    1

#define AS_IMAGE_NOT_IN_FLASH          -1

#define AS_FILE_BUFFER_SIZE AS_FLASH_SECTOR_SIZE
#define AS_SDCARD_APP_DIRECTORY "Altera_NEEK_Applications"

typedef struct {
  unsigned int image_offset;
  unsigned char type;
  int image_length;
  unsigned int image_crc;
  int progress; //download progress
  char hw_filename[80];
  char sw_filename[80];
  unsigned int directory;
  char app_name[80];
} app_info;

typedef struct {
  app_info apps[AS_MAX_SDCARD_APPS];
  int num_apps;
  int target_app;
  int action;  
} app_list_struct;


//function
int AsFindAppsOnSD( app_list_struct* sd_app_list );
void Down_SW_BIN(app_list_struct* sd_app_list , alt_video_display* display);
int Down_HW_BIN(app_list_struct* sd_app_list, alt_video_display* display);
void ComputeTotalSize(app_list_struct* sd_app_list);
void Display_Process(app_list_struct* sd_app_list , alt_video_display* display);
int EPCS_Reconfig( int remote_update_base,int flash_base,int reconfig_offset,int watchdog_timeout,int width_of_flash );
int Check_IsExsist(app_list_struct* sd_app_list , int demo_address);
#endif /*APP_SELECTOR_H_*/
